/*	Public domain	*/

#ifdef HAVE_PERL
#include <EXTERN.h>
#include <perl.h>
#endif

#include <sys/types.h>
#include <sys/file.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>

#include <config/have_setproctitle.h>
#include <config/have_flock.h>
#include <config/have_open_exlock.h>

#include <config/have_bounded_attribute.h>
#include <config/have_format_attribute.h>
#include <config/have_nonnull_attribute.h>
#include <config/have_packed_attribute.h>

#ifdef HAVE_BOUNDED_ATTRIBUTE
#define BOUNDED_ATTRIBUTE(t, a, b) __attribute__((__bounded__ (t,a,b)))
#else
#define BOUNDED_ATTRIBUTE(t, a, b)
#endif
#ifdef HAVE_FORMAT_ATTRIBUTE
#define FORMAT_ATTRIBUTE(t, a, b) __attribute__((__format__ (t,a,b)))
#else
#define FORMAT_ATTRIBUTE(t, a, b)
#endif
#ifdef HAVE_NONNULL_ATTRIBUTE
#define NONNULL_ATTRIBUTE(a) __attribute__((__nonnull__ (a)))
#else
#define NONNULL_ATTRIBUTE(a)
#endif
#ifdef HAVE_PACKED_ATTRIBUTE
#define PACKED_ATTRIBUTE __attribute__((__packed__))
#else
#define PACKED_ATTRIBUTE
#endif

#include "queue.h"
#include "table.h"
#include "fgetln.h"
#include "strlcpy.h"
#include "strlcat.h"

#include "sockunion.h"
#include "maillist.h"
#include "message.h"
#include "sa.h"
#include "smtp.h"
#include "qmgr.h"
#include "local.h"
#include "policy.h"
#include "control.h"
#include "spamcheck.h"

#define Malloc(len) SF_Malloc(len)
#define Free(p) if ((p) != NULL) free(p)
#define Fatal SF_Fatal

#ifdef DEBUG
#define Debug SF_Debug
#else
#define Debug(arg...) ((void)0)
#endif

#ifdef HAVE_SETPROCTITLE
#define Setproctitle setproctitle
#else
#define Setproctitle(arg...) ((void)0)
#endif

#if defined(__OpenBSD__) || defined(__NetBSD__) || defined(__FreeBSD__)
typedef socklen_t my_socklen_t;
#else
typedef int my_socklen_t;
#endif

typedef enum {
	SF_NONE,
	SF_AND,
	SF_OR
} SF_LogicOp;

extern int sfSoftFail;
extern int sfPollInterval;
extern int sfPolicyServer;
extern char *sfHostSMTP, *sfPortSMTP;
extern char *sfReinHost, *sfReinPort;
extern FILE *sfLog;

void	 SF_Debug(const char *, ...);
void	*SF_Malloc(size_t);
void	 SF_Fatal(const char *, ...);
void	 SF_SetError(const char *, ...);
char	*SF_GetError(void);
void	 SF_EnterServerProc(const char *);
void	 SF_ExitServerProc(int);

#ifdef HAVE_FLOCK
#define LockFile(fd) flock((fd),LOCK_EX)
#define UnlockFile(fd) flock((fd),LOCK_UN)
#else
#define LockFile(fd) (void)(0)
#define UnlockFile(fd) (void)(0)
#endif

#define Unlink(path) do { \
	if (unlink(path) == -1) \
		syslog(LOG_ERR, "unlink %s: %s", path, strerror(errno)); \
} while(0)

static __inline__ int
Write(int fd, const void *data, size_t len)
{
	size_t nwrote;
	ssize_t rv;

	for (nwrote = 0; nwrote < len; ) {
		rv = write(fd, data+nwrote, len-nwrote);
		if (rv == -1) {
			if (errno == EINTR) {
				continue;
			} else {
				SF_SetError("Write error: %s", strerror(errno));
				return (-1);
			}
		} else if (rv == 0) {
			SF_SetError("EOF");
			return (-1);
		}
		nwrote += rv;
	}
	return (0);
}

static __inline__ int
Read(int fd, void *data, size_t len)
{
	size_t nread;
	ssize_t rv;

	for (nread = 0; nread< len; ) {
		rv = read(fd, data+nread, len-nread);
		if (rv == -1) {
			if (errno == EINTR) {
				continue;
			} else {
				SF_SetError("Read error: %s", strerror(errno));
				return (-1);
			}
		} else if (rv == 0) {
			SF_SetError("EOF");
			return (-1);
		}
		nread += rv;
	}
	return (0);
}
